;****************************************************************
;
; MegaBlast - MSX ver 1.00 (C) Electric Adventures 2014-2017
;
;****************************************************************
FNAME "MEGABLAST.ROM"
cpu z80
ORG 4000h

INCLUDE "MSXRom-Include.asm"

PATSIZE:   EQU 71
SPRITECOUNT: EQU 11

db "AB"
DW INIT
DW 0
DW 0
DW 0
DS 6

; This is the start of your application
INIT:
    ; set screen mode immediately, as interrupts are enabled on entry
    call SETSCREEN2

    ; initialise clock
    LD  HL,TIMER_TABLE
    LD  DE,TIMER_DATA_BLOCK
    CALL INIT_TIMER

    ;Enable timers - Tick, 1/4 sec and 1/2 sec timers
    CALL CREATE_TIMERS

    CALL SETINTERRUPT

    ; Seed random numbers with a fixed number (nothing else to use?)
    LD HL,1967
    CALL SEED_RANDOM


    ; Do all our VRAM setup
    ; NMI is currently disabled

TITLESCREEN:
    ; display our title screen
    DI
    ; Clear the screen
    CALL CLEARPAT

    ; Load the character set, make all three sections the same
    CALL LOAD_CHR_SET

    ; now setup the title screen layout
    LD HL,VRAM_NAME
    LD DE,TITLE_SCREEN_PAT
    LD BC,24*32
    CALL LDIRVM

   	LD HL,OUTPUT_VDP_TITLE
	CALL SET_VDU_HOOK
    EI
    LD A,15
    LD (LEVEL),A
SPLASH_TITLE2:
	CALL JOYTST
	CP 255
	JR Z,NGAME
    LD	A,(HalfSecTimer)
	CALL	TEST_SIGNAL
	OR	A
    JR Z,SPLASH_TITLE2
    ; Any other actions on the title screen go here
	JR SPLASH_TITLE2

NGAME:
    DI
    CALL INITRAM

    ; Send the sprite definitions to the VDP
    LD HL,VRAM_SPRGEN
    LD DE,MAIN_SHIP
    LD BC,32*SPRITECOUNT
    CALL LDIRVM

    ; Load the character set, make all three sections the same
    CALL LOAD_CHR_SET

    ; now setup the title screen layout
    LD HL,VRAM_NAME
    LD DE,MAINLAYOUT
    LD BC,24*32
    CALL LDIRVM

MAIN_SCREEN:
    ; Read joysticks to clear any false reads
    CALL JOYTST

	LD HL,(TIME)
	LD (SEED),HL
	RR H
	RL L
	LD (SEED+2),HL

	; disable interrupts
    DI

    ; Clean up in case the game left anything on screen
	CALL CLEARSPRITES
	CALL SPRWRT

    ; now setup the main screen layout
	LD HL,VRAM_NAME
	LD DE,MAINLAYOUT
	LD BC,24*32
	CALL LDIRVM

    ; set position of player ship
    LD A,150
    LD (SPRTBL),A
    LD (SPRTBL+4),A
    LD A,120
    LD (SPRTBL+1),A
    LD (SPRTBL+5),A
    XOR A
    LD (SPRTBL+2),A
    LD A,4
    LD (SPRTBL+6),A
    LD A,05h
    LD (SPRTBL+3),A
    LD A,0fh
    LD (SPRTBL+7),A

    CALL DISPLAYLIVES

    LD A,1
    LD (LASTSCORE),A
    CALL DISPLAYSCORE

	LD HL,VDU_WRITES
	CALL SET_VDU_HOOK
	EI

    ; Main game logic loop
MLOOP:
    ; check that a base tick has occurred
    ; ensures consistent movement speed between 50 & 60Hz systems
	LD	A,(TickTimer)
	CALL	TEST_SIGNAL
	OR	A
	JR Z,MLOOP2
    ; once per tick
    CALL MOVE_PLAYER
    CALL FIRE_PLAYER_BULLET
    CALL MOVE_PLAYER_BULLET

MLOOP2:
    LD A,(QtrSecTimer)
    CALL TEST_SIGNAL
    OR A
    JR Z,MLOOP

    JR MLOOP


; Check for player bullet firing
FIRE_PLAYER_BULLET:
    ; make sure there is not already a bullet
    LD A,(SPRTBL+8)
    CP 209
    RET NZ
    ; see if the fire button is pressed
    CALL JOYTST
    CP 255
    RET NZ
    ; fire bullet
    ; set Y based on player ship
    LD A,(SPRTBL)
    SUB A,6
    LD (SPRTBL+8),A
    ; set X based on player ship
    LD A,(SPRTBL+1)
    ADD A,6
    LD (SPRTBL+9),A
    ; set bullet sprite pattern
    LD A,24
    LD (SPRTBL+10),A
    ; set bullet colour
    LD A,11
    LD (SPRTBL+11),A
    RET

; Move the players bullet
MOVE_PLAYER_BULLET:
    ; check that the bullet is visible
    LD A,(SPRTBL+8)
    CP 209
    RET Z
    ; decrease bullets Y position
    DEC A
    DEC A
    DEC A
    CP 4
    JR NC, MPB1
    ; bullet has reached the top of the screen, hide the bullet
    LD A,209
MPB1:
    ; save new position
    LD (SPRTBL+8),A
    RET

; Detect joystick direction and move the player accordingly
MOVE_PLAYER:
    CALL JOYDIR
    CP 3
    JR NZ,NRIGHT
    ; move to the right
    LD A,(SPRTBL+1)
    CP 242
    JR NC,NLEFT
    INC A
    LD (SPRTBL+1),A
    LD (SPRTBL+5),A
    JR NLEFT
NRIGHT:
    CP 7
    JR NZ,NLEFT
    ; move to the left
    LD A,(SPRTBL+1)
    CP 0
    JR Z,NLEFT
    DEC A
    LD (SPRTBL+1),A
    LD (SPRTBL+5),A
NLEFT:
    RET



; Load the character set, make all three sections the same
LOAD_CHR_SET:
    LD HL,0
SLOOP:
    LD DE,TILESET_1_PAT
    PUSH HL
    LD BC,PATSIZE*8
    CALL LDIRVM
    POP HL
    ; now load colour attributes
    PUSH HL
    LD BC,VRAM_COLOR
    ADD HL,BC
    LD DE,TILESET_1_COL
    LD BC,PATSIZE*8
    CALL LDIRVM
    POP HL
    LD BC,800h
    ADD HL,BC
    LD A,H
    CP 18h
    JR C,SLOOP
    RET


; Init Ram for a new game
INITRAM:
    LD A,3
    LD (LIVES),A
    LD HL,0
    LD (SCORE),HL
    LD (SCORE+1),HL
    LD A,1
    LD (LASTSCORE),A
    XOR A
    LD (LEVEL),A
    RET

; Display the current player lives (max 6)
DISPLAYLIVES:
    ; clear current lives display
    LD HL,VRAM_NAME+688
    LD BC,12
    XOR A
    CALL FILVRM
    LD HL,VRAM_NAME+720
    LD BC,12
    XOR A
    CALL FILVRM
    ; now show the current lives
    ; - first write the top characters
    LD D,67
    LD HL,VRAM_NAME+688
    CALL DL2
    ; - now write the bottom characters
    LD D,69
    LD HL,VRAM_NAME+720
DL2:
    CALL SETWRT
    LD A,(LIVES)
    LD B,A
DL1:
    LD A,D
    OUT (DATA_PORT),A
    INC A
    OUT (DATA_PORT),A
    DJNZ DL1
    RET

; Display the current score on the 2nd last row
DISPLAYSCORE:
	LD A,(SCORE)
	LD HL,LASTSCORE
	CP (HL)
	RET Z
	LD HL,VRAM_NAME + 707
	CALL SETWRT
	LD HL,SCORE+2
	LD B,3
SLP:
    CALL PRINTIT
	DEC HL
	DEC B
	LD A,B
	CP 0
	JR NZ,SLP
	LD A,(SCORE)
	LD (LASTSCORE),A
    RET

; This is our routine called every VDP interrupt during normal game play
; - Do all VDP writes here to avoid corruption
VDU_WRITES:
    CALL DISPLAYSCORE
    RET

; This is our routine called every VDP interrupt during the title screen
; - Do all VDP writes here to avoid corruption
OUTPUT_VDP_TITLE:
    RET


TITLE_SCREEN_PAT:
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,023,000,015,000,017,000,011,000,000,000,000
    DB 012,000,022,000,011,000,029,000,030,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 037,038,039,040,037,038,039,040,037,038,039,040,037,038,039,040
    DB 037,038,039,040,037,038,039,040,037,038,039,040,037,038,039,040
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    ; press fire to begin
    DB 000,000,000,000,000,000,026,028,015,029,029,000,016,019,028,015
    DB 000,030,025,000,012,015,017,019,024,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    ; EA Logo
    DB 061,061,061,061,061,061,061,061,061,061,061,057,058,059,060,061
    DB 062,063,064,065,066,061,061,061,061,061,061,061,061,061,061,061

MAINLAYOUT:
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 037,038,039,040,037,038,039,040,037,038,039,040,037,038,039,040
    DB 037,038,039,040,037,038,039,040,037,038,039,040,037,038,039,040
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    ; planet surface
    DB 041,041,041,041,041,041,041,041,041,041,041,041,041,041,041,041
    DB 041,041,041,041,041,041,041,041,041,041,041,041,041,041,041,041
    ; score and life area
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    DB 000,000,000,001,001,001,001,001,001,001,000,000,000,000,000,000
    DB 000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000
    ; EA Logo
    DB 061,061,061,061,061,061,061,061,061,061,061,057,058,059,060,061
    DB 062,063,064,065,066,061,061,061,061,061,061,061,061,061,061,061


;**************************************************************************************************
; Include external data files
;**************************************************************************************************

include "MSXRom-MegaBlast-Patterns.ASM"


;**************************************************************************************************
; Standard Libraries
;**************************************************************************************************

include "MSXRom-Lib.ASM"

END:	EQU $

;**************************************************************************************************
; RAM Definitions
;**************************************************************************************************

ORG RAMSTART

LEVEL:	     DS 1
LIVES:       DS 1
SCORE:	     DS 3
LASTSCORE:   DS 3



